@@ -8,8 +8,10 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package MooseX::AttributeShortcuts;
+# git description: 0.018-7-gdc99f4b
+
 {
-  $MooseX::AttributeShortcuts::VERSION = '0.017';
+  $MooseX::AttributeShortcuts::VERSION = '0.019';
 }
 
 # ABSTRACT: Shorthand for common attribute options
@@ -29,8 +31,10 @@ use Moose::Util::TypeConstraints;
 
 {
     package MooseX::AttributeShortcuts::Trait::Attribute;
+# git description: 0.018-7-gdc99f4b
+
 {
-  $MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.017';
+  $MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.019';
 }
     use namespace::autoclean;
     use MooseX::Role::Parameterized;
@@ -53,6 +57,8 @@ use Moose::Util::TypeConstraints;
     role {
         my $p = shift @_;
 
+        with 'MooseX::CoercePerAttribute' => { -version => 0.802 };
+
         my $wprefix = $p->writer_prefix;
         my $bprefix = $p->builder_prefix;
         my %prefix = (
@@ -126,26 +132,35 @@ use Moose::Util::TypeConstraints;
             $options->{isa} = enum(delete $options->{isa_enum})
                 if $_has->('isa_enum');
 
-            $class->throw_error('You must specify an isa when declaring a constraint')
-                if $_has->('constraint') && !$_has->('isa');
+            ### the pretty business of on-the-fly subtyping...
+            if ($_has->('constraint')) {
 
-            if (my $isa = $_opt->('isa')) {
-
-                my @opts;
+                # check for errors...
+                $class->throw_error('You must specify an "isa" when declaring a "constraint"')
+                    if !$_has->('isa');
+                $class->throw_error('"constraint" must be a CODE reference')
+                    if $_ref->('constraint') ne 'CODE';
 
                 # constraint checking! XXX message, etc?
-                push @opts, constraint => $_opt->('constraint')
+                push my @opts, constraint => $_opt->('constraint')
                     if $_ref->('constraint') eq 'CODE';
 
-                if (@opts) {
+                # stash our original option away and construct our new one
+                my $isa = $options->{original_isa} = $_opt->('isa');
+                my $isa_tc = _find_or_create_isa_type_constraint($isa);
+                my $new_tc = $isa_tc->create_child_type(@opts);
 
-                    # stash our original option away and construct our new one
-                    $options->{original_isa} = $isa;
-                    $options->{isa}
-                        = _find_or_create_isa_type_constraint($isa)
-                        ->create_child_type(@opts)
-                        ;
+                if ($isa_tc->has_coercion && $_opt->('coerce') eq "1") {
+
+                    # create our coercion as a copy of the parent
+                    $new_tc->coercion(Moose::Meta::TypeCoercion->new(
+                        type_constraint   => $new_tc,
+                        type_coercion_map => $isa_tc->coercion->type_coercion_map,
+                    ));
                 }
+
+                # fin constraint mucking....
+                $options->{isa} = $new_tc;
             }
 
             if ($options->{lazy_build} && $options->{lazy_build} eq 'private') {
@@ -296,7 +311,7 @@ __END__
 
 =encoding utf-8
 
-=for :stopwords Chris Weyl
+=for :stopwords Chris Weyl GitHub attribute's isa one's rwp SUBTYPING
 
 =head1 NAME
 
@@ -304,7 +319,7 @@ MooseX::AttributeShortcuts - Shorthand for common attribute options
 
 =head1 VERSION
 
-This document describes version 0.017 of MooseX::AttributeShortcuts - released October 28, 2012 as part of MooseX-AttributeShortcuts.
+This document describes version 0.019 of MooseX::AttributeShortcuts - released April 20, 2013 as part of MooseX-AttributeShortcuts.
 
 =head1 SYNOPSIS
 
@@ -357,6 +372,8 @@ This package causes an attribute trait to be applied to all attributes defined
 to the using class.  This trait extends the attribute option processing to
 handle the above variations.
 
+=for Pod::Coverage init_meta
+
 =head1 USAGE
 
 This package automatically applies an attribute metaclass trait.  Unless you
@@ -497,13 +514,14 @@ e.g., in your class,
     has foo => (is => 'ro', builder => '_build_foo');
     sub _build_foo { 'bar!' }
 
-=head2 constraint => sub { ... }
+=head2 isa => ..., constraint => sub { ... }
 
-Specifying the constraint option with a coderef will cause a new type
+Specifying the constraint option with a coderef will cause a new subtype
 constraint to be created, with the parent type being the type specified in the
 C<isa> option and the constraint being the coderef supplied here.
 
-Example:
+For example, only integers greater than 10 will pass this attribute's type
+constraint:
 
     # value must be an integer greater than 10 to pass the constraint
     has thinger => (
@@ -514,7 +532,68 @@ Example:
 
 Note that if you supply a constraint, you must also provide an C<isa>.
 
-=for Pod::Coverage init_meta
+=head2 isa => ..., constraint => sub { ... }, coerce => 1
+
+Supplying a constraint and asking for coercion will "Just Work", that is, any
+coercions that the C<isa> type has will still work.
+
+For example, let's say that you're using the C<File> type constraint from
+L<MooseX::Types::Path::Class>, and you want an additional constraint that the
+file must exist:
+
+    has thinger => (
+        is         => 'ro',
+        isa        => File,
+        constraint => sub { !! $_->stat },
+        coerce     => 1,
+    );
+
+C<thinger> will correctly coerce the string "/etc/passwd" to a
+C<Path::Class:File>, and will only accept the coerced result as a value if
+the file exists.
+
+=head2 coerce => { Type => sub { ...coerce... }, ... }
+
+Specifying the coerce option with a hashref will cause a new subtype to be
+created and used (just as with the constraint option, above), with the
+specified coercions added to the list.  In the passed hashref, the keys are
+Moose types (well, strings resolvable to Moose types), and the values are
+coderefs that will coerce a given type to our type.
+
+    has bar => (
+        is     => 'ro',
+        isa    => 'Str',
+        coerce => {
+            Int    => sub { "$_"                       },
+            Object => sub { 'An instance of ' . ref $_ },
+        },
+    );
+
+=head1 ANONYMOUS SUBTYPING AND COERCION
+
+Note that we create new, anonymous subtypes whenever the constraint or
+coercion options are specified in such a way that the Shortcuts trait (this
+one) is invoked.  It's fully supported to use both constraint and coerce
+options at the same time.
+
+This facility is intended to assist with one-off type constraints and
+coercions.  It is not possible to deliberately reuse the subtypes we create,
+and if you find yourself using a particular isa / constraint / coerce option
+triplet in more than one place you should really think about creating a type
+that you can reuse.  L<MooseX::Types> provides the facilities to easily do
+this.
+
+=head1 SEE ALSO
+
+Please see those modules/websites for more information related to this module.
+
+=over 4
+
+=item *
+
+L<L<MooseX::CoercePerAttribute> provides our subtype coercion support|L<MooseX::CoercePerAttribute> provides our subtype coercion support>
+
+=back
 
 =head1 SOURCE
 
